import { useForm, Controller } from "react-hook-form";
import { Box, Button, Typography, Modal } from "@mui/material";
import CloseIcon from "@mui/icons-material/Close";
import { CustomIconButton, CustomTextField } from "../AddUserModal/styled.css";
import { useDispatch } from "react-redux";
import { addVehicle } from "../../redux/slices/VehicleSlice";
import { autofillFix } from "../../shared-theme/customizations/autoFill";
import { useState } from "react";

interface FormData {
	name: string;
	company: string;
	modelName: string;
	chargeType: string;
	imageFile: File | null;
}

interface AddVehicleModalProps {
	open: boolean;
	handleClose: () => void;
}

export default function AddVehicleModal({
	open,
	handleClose,
}: AddVehicleModalProps) {
	const {
		control,
		handleSubmit,
		formState: { errors },
		reset,
		setValue,
	} = useForm<FormData>({
		defaultValues: {
			name: "",
			company: "",
			modelName: "",
			chargeType: "",
			imageFile: null,
		},
	});

	const dispatch = useDispatch();
	const [imagePreview, setImagePreview] = useState<string | null>(null);

	const handleImageChange = (e: React.ChangeEvent<HTMLInputElement>) => {
		const file = e.target.files?.[0];
		if (file) {
			setImagePreview(URL.createObjectURL(file)); // Preview the image
			setValue("imageFile", file); // Set the file in the form
		}
	};

	const onSubmit = (data: FormData) => {
		if (!data.imageFile) {
			console.error("No file selected");
			return;
		}

		dispatch(
			addVehicle({
				name: data.name,
				company: data.company,
				modelName: data.modelName,
				chargeType: data.chargeType,
				imageFile: data.imageFile,
			})
		);

		handleClose();
		reset();
		setImagePreview(null); // Clear image preview
	};

	return (
		<Modal
			open={open}
			onClose={(e, reason) => {
				if (reason === "backdropClick") {
					return;
				}
				handleClose();
			}}
			aria-labelledby="add-vehicle-modal"
		>
			<Box
				component="form"
				onSubmit={handleSubmit(onSubmit)}
				sx={{
					position: "absolute",
					top: "50%",
					left: "50%",
					transform: "translate(-50%, -50%)",
					width: 400,
					bgcolor: "background.paper",
					boxShadow: 24,
					p: 3,
					borderRadius: 2,
				}}
			>
				{/* Header */}
				<Box
					sx={{
						display: "flex",
						justifyContent: "space-between",
						alignItems: "center",
					}}
				>
					<Typography variant="h6" fontWeight={600}>
						Add Vehicle
					</Typography>
					<CustomIconButton onClick={handleClose}>
						<CloseIcon />
					</CustomIconButton>
				</Box>

				{/* Horizontal Line */}
				<Box sx={{ borderBottom: "1px solid #ddd", my: 2 }} />

				{/* Input Fields */}
				<Box sx={{ display: "flex", flexDirection: "column", gap: 2 }}>
					{/* First Row - Two Inputs */}
					<Box sx={{ display: "flex", gap: 2 }}>
						<Box
							sx={{
								display: "flex",
								flexDirection: "column",
								width: "100%",
								...autofillFix,
							}}
						>
							<Typography
								variant="body2"
								fontWeight={500}
								mb={0.5}
							>
								Vehicle Name
							</Typography>
							<Controller
								name="name"
								control={control}
								rules={{
									required: "Vehicle Name is required",
									minLength: {
										value: 3,
										message:
											"Minimum 3 characters required",
									},
									maxLength: {
										value: 30,
										message:
											"Maximum 30 characters allowed",
									},
									pattern: {
										value: /^[A-Za-z\s]+$/,
										message:
											"Vehicle Name must only contain letters and spaces",
									},
								}}
								render={({ field }) => (
									<CustomTextField
										{...field}
										fullWidth
										placeholder="Enter Vehicle Name"
										size="small"
										sx={{ marginTop: 1 }}
										error={!!errors.name}
										helperText={errors.name?.message}
									/>
								)}
							/>
						</Box>

						<Box
							sx={{
								display: "flex",
								flexDirection: "column",
								width: "100%",
								...autofillFix,
							}}
						>
							<Typography
								variant="body2"
								fontWeight={500}
								mb={0.5}
							>
								Company
							</Typography>
							<Controller
								name="company"
								control={control}
								rules={{
									required: "Company is required",
									minLength: {
										value: 3,
										message:
											"Minimum 3 characters required",
									},
								}}
								render={({ field }) => (
									<CustomTextField
										{...field}
										fullWidth
										placeholder="Enter Company Name"
										size="small"
										sx={{ marginTop: 1 }}
										error={!!errors.company}
										helperText={errors.company?.message}
									/>
								)}
							/>
						</Box>
					</Box>

					{/* Second Row - Two Inputs */}
					<Box sx={{ display: "flex", gap: 2 }}>
						<Box
							sx={{
								display: "flex",
								flexDirection: "column",
								width: "100%",
								...autofillFix,
							}}
						>
							<Typography
								variant="body2"
								fontWeight={500}
								mb={0.5}
							>
								Model Name
							</Typography>
							<Controller
								name="modelName"
								control={control}
								rules={{ required: "Model Name is required" }}
								render={({ field }) => (
									<CustomTextField
										{...field}
										fullWidth
										placeholder="Enter Model Name"
										size="small"
										sx={{ marginTop: 1 }}
										error={!!errors.modelName}
										helperText={errors.modelName?.message}
									/>
								)}
							/>
						</Box>

						<Box
							sx={{
								display: "flex",
								flexDirection: "column",
								width: "100%",
								...autofillFix,
							}}
						>
							<Typography
								variant="body2"
								fontWeight={500}
								mb={0.5}
							>
								Charge Type
							</Typography>
							<Controller
								name="chargeType"
								control={control}
								rules={{ required: "Charge Type is required" }}
								render={({ field }) => (
									<CustomTextField
										{...field}
										fullWidth
										placeholder="Enter Charge Type"
										size="small"
										sx={{ marginTop: 1 }}
										error={!!errors.chargeType}
										helperText={errors.chargeType?.message}
									/>
								)}
							/>
						</Box>
					</Box>

					{/* Image Upload */}
					<Box
						sx={{
							display: "flex",
							flexDirection: "column",
							width: "100%",
							...autofillFix,
						}}
					>
						<Typography variant="body2" fontWeight={500} mb={0.5}>
							Upload Image
						</Typography>
						<Button
							component="label"
							sx={{
								backgroundColor: "#52ACDF",
								color: "white",
								borderRadius: "8px",
								width: "100%",
								"&:hover": { backgroundColor: "#52ACDF" },
							}}
						>
							Choose Image
							<input
								type="file"
								hidden
								accept="image/*"
								onChange={handleImageChange}
							/>
						</Button>
						{errors.imageFile && (
							<Typography
								variant="caption"
								color="error"
								sx={{ mt: 1 }}
							>
								{errors.imageFile.message}
							</Typography>
						)}
						{imagePreview && (
							<Box sx={{ marginTop: 2 }}>
								<Typography variant="body2" mb={1}>
									Preview:
								</Typography>
								<img
									src={imagePreview}
									alt="image preview"
									style={{
										maxWidth: "100%",
										height: "auto",
										borderRadius: "8px",
									}}
								/>
							</Box>
						)}
					</Box>
				</Box>

				{/* Submit Button */}
				<Box
					sx={{ display: "flex", justifyContent: "flex-end", mt: 3 }}
				>
					<Button
						type="submit"
						sx={{
							backgroundColor: "#52ACDF",
							color: "white",
							borderRadius: "8px",
							width: "117px",
							"&:hover": { backgroundColor: "#439BC1" },
						}}
					>
						Add Vehicle
					</Button>
				</Box>
			</Box>
		</Modal>
	);
}
